@@ -1,29 +1,27 @@
-# Copyright 2009-2010 Ram Rachum. No part of this program may be used, copied
+# Copyright 2009-2011 Ram Rachum. No part of this program may be used, copied
 # or distributed without explicit written permission from Ram Rachum.
 
 '''
-Defines the Frame class.
+Defines the `Frame` class.
 
 See its documentation for more information.
 '''
 
 from __future__ import with_statement
 
-import os
+import os.path
 import sys
-import random
-import cPickle as pickle_module
 import subprocess
-import webbrowser
+import warnings
 import traceback
 
 import wx
 from garlicsim_wx.general_misc.third_party import aui
 import pkg_resources
 
-from garlicsim.general_misc import dict_tools
-from garlicsim.general_misc import string_tools
+from garlicsim.general_misc.temp_value_setters import TempRecursionLimitSetter
 from garlicsim_wx.general_misc import thread_timer
+from garlicsim_wx.general_misc import misc_tools
 from garlicsim_wx.general_misc import wx_tools
 
 import garlicsim
@@ -31,30 +29,49 @@ from garlicsim_wx.gui_project import GuiProject
 import garlicsim_wx.widgets
 import garlicsim_wx.misc
 from garlicsim_wx.widgets import workspace_widgets
+from garlicsim_wx import misc
 
 from . import images as __images_package
 images_package = __images_package.__name__
 
 
+wildcard_text = ('GarlicSim Simulation Pickle (*.gssp)|*.gssp|'
+                 'All files (*)|*')
+
+
 class Frame(wx.Frame):
     '''
-    The main window of garlicsim_wx.
+    The main window of `garlicsim_wx`.
     
     This window allows the user to create and manipulate gui projects.
     '''
-    def __init__(self, *args, **keywords):
+    def __init__(self, *args, **kwargs):
+        wx.Frame.__init__(self, *args, **kwargs)
         
-        wx.Frame.__init__(self, *args, **keywords)
+        self.SetBackgroundStyle(wx.BG_STYLE_CUSTOM | wx.BG_STYLE_COLOUR)
+        self.SetBackgroundColour(wx_tools.get_background_color())
         
         self.SetDoubleBuffered(True)
         self.SetIcons(garlicsim_wx.misc.icon_bundle.get_icon_bundle())
         
-        self.Bind(wx.EVT_CLOSE, self.on_close)        
+        self.Bind(wx.EVT_CLOSE, self.on_close)
+        
+        self.app = wx.GetApp()
+        '''The GarlicSim `App` that created this frame.'''
+        
+        assert isinstance(self.app, garlicsim_wx.App)
+        
+        if hasattr(garlicsim_wx, '_active_frame'):
+            warnings.warn('`garlicsim_wx` already had an `_active_frame` '
+                          'attribute before we created this frame! It should '
+                          'not have one.')
+        garlicsim_wx._active_frame = self
         
         self.tree_browser = None
         self.seek_bar = None
         self.shell = None
         self.state_repr_viewer = None
+        self.crunching_controls = None
         
         self.aui_manager = garlicsim_wx.misc.aui.AuiManager(self)
         '''The aui manager, which manages the workspace widgets.'''
@@ -103,8 +120,9 @@ class Frame(wx.Frame):
         try_recalculate = lambda thing: \
             thing._recalculate() if hasattr(thing, '_recalculate') else None
         
-        menus_to_recalculate = [menu for (menu, label) in                                 
-                                self.menu_bar.GetMenus()]
+        menus_to_recalculate = \
+            [menu for (menu, label) in self.menu_bar.GetMenus()]
+        # todo: Should be calculating context menu too?
         
         while menus_to_recalculate:
             menu = menus_to_recalculate.pop()
@@ -146,7 +164,8 @@ class Frame(wx.Frame):
                 
         def on_up():
             '''Go one path upwards.'''
-            if self.gui_project.path and self.gui_project.active_node:
+            if self.gui_project and self.gui_project.path and \
+               self.gui_project.active_node:
                 try:
                     new_path = self.gui_project.path._get_lower_path(
                         self.gui_project.active_node
@@ -161,7 +180,8 @@ class Frame(wx.Frame):
                 
         def on_down():
             '''Go one path downwards.'''
-            if self.gui_project.path and self.gui_project.active_node:
+            if self.gui_project and self.gui_project.path and \
+               self.gui_project.active_node:
                 try:
                     new_path = self.gui_project.path._get_higher_path(
                         self.gui_project.active_node
@@ -228,7 +248,8 @@ class Frame(wx.Frame):
             if self.gui_project is not None and \
                self.gui_project.active_node is not None:
                 
-                node = self.gui_project.active_node.get_ancestor(20, round=True)
+                node = self.gui_project.active_node.get_ancestor(20,
+                                                                 round=True)
                 if node:
                     self.gui_project.set_active_node(node)
         
@@ -256,7 +277,10 @@ class Frame(wx.Frame):
                 
         def on_return():
             '''Finalize the active node, if it's in editing.'''
-            if self.gui_project and self.gui_project.active_node.still_in_editing:
+
+            if self.gui_project and self.gui_project.active_node \
+               and self.gui_project.active_node.still_in_editing:
+                
                 self.gui_project.finalize_active_node()
 
         
@@ -285,7 +309,7 @@ class Frame(wx.Frame):
         self.aui_manager.UnInit()
         self.Destroy()
         garlicsim_wx.general_misc.cute_base_timer.CuteBaseTimer.\
-                    stop_timers_by_frame(self)
+            stop_timers_by_frame(self)
         event.Skip()        
         self.background_timer.stop()
 
@@ -308,22 +332,28 @@ class Frame(wx.Frame):
                 main_script = os.path.abspath(sys.argv[0])
                 program_to_run = [sys.executable, main_script]
                 we_are_main_program = ('run_gui' in main_script) or \
-                                    ('garlicsim_wx' in main_script)
+                                    ('garlicsim_wx' in main_script) or \
+                                    ('GarlicSim' in main_script)
             
             if not we_are_main_program:
                 warning_dialog = \
                     garlicsim_wx.widgets.misc.NotMainProgramWarningDialog(self)
-                if warning_dialog.ShowModal() != wx.ID_YES:
-                    return
+                try:
+                    if warning_dialog.ShowModal() != wx.ID_YES:
+                        return
+                finally:
+                    warning_dialog.Destroy()
         
         dialog = garlicsim_wx.widgets.misc.SimpackSelectionDialog(self)
         
-        if dialog.ShowModal() == wx.ID_OK:
-            simpack = dialog.get_simpack_selection()
-        else:
+        try:
+            if dialog.ShowModal() == wx.ID_OK:
+                simpack = dialog.get_simpack_selection()
+            else:
+                dialog.Destroy()
+                return
+        finally:
             dialog.Destroy()
-            return
-        dialog.Destroy()
 
         
         if self.gui_project is None:
@@ -331,8 +361,12 @@ class Frame(wx.Frame):
         else:    
                 
             program_to_run.append('__garlicsim_wx_new=%s' % simpack.__name__)
+            for simpack_place in garlicsim_wx.simpack_places:
+                program_to_run.append('__garlicsim_wx_simpack_place=%s' % \
+                                      ','.join(simpack_place))
          
-            subprocess.Popen(program_to_run)
+            with wx_tools.CursorChanger(self, wx.CURSOR_WAIT):
+                subprocess.Popen(program_to_run)
             
             return
             
@@ -343,8 +377,9 @@ class Frame(wx.Frame):
         Internal use.
         '''
         assert self.gui_project is None
-        gui_project = GuiProject(simpack, self)
-        self.__setup_gui_project(gui_project)
+        with wx_tools.CursorChanger(self, wx.CURSOR_WAIT):
+            gui_project = GuiProject(simpack, self)
+            self.__setup_gui_project(gui_project)
 
         
     def on_exit_menu_button(self, event):
@@ -381,6 +416,8 @@ class Frame(wx.Frame):
         
         return nodes_added
     
+    
+    
     def __setup_gui_project(self, gui_project):
         '''
         Setup a newly-created gui project.
@@ -388,118 +425,151 @@ class Frame(wx.Frame):
         Internal use.
         '''
         
+        assert isinstance(gui_project, GuiProject)
+        
         self.gui_project = gui_project
         
         # todo: should create StateReprViewer only if the simpack got no
         # workspace widgets
         
-        self.tree_browser = workspace_widgets.TreeBrowser(self)
-        self.aui_manager.AddPane(
-            self.tree_browser,
-            aui.AuiPaneInfo()\
-            .Bottom().Row(0)\
-            .BestSize(1000, 100).MinSize(200, 50).MaxSize(10000, 250)\
-            .Caption(self.tree_browser.get_uppercase_name())
-            .Floatable(False)\
-            .CloseButton(False)
-        )
-        
-        self.playback_controls = workspace_widgets.PlaybackControls(self)
-        self.aui_manager.AddPane(
-            self.playback_controls,
-            aui.AuiPaneInfo()\
-            .Bottom()\
-            .BestSize(184, 128).MinSize(184, 128).MaxSize(184, 128)\
-            .Caption(self.playback_controls.get_uppercase_name())
-            .Resizable(False)\
-            .CloseButton(False)        
-        )
-        
-        self.seek_bar = workspace_widgets.SeekBar(self)
-        self.aui_manager.AddPane(
-            self.seek_bar,
-            aui.AuiPaneInfo()\
-            .Bottom().Row(1)\
-            .BestSize(600, 40).MinSize(200, 40).MaxSize(10000, 100)\
-            .Caption(self.seek_bar.get_uppercase_name())
-            .Floatable(False)\
-            .CloseButton(False)
-        )
-        
-        self.shell = workspace_widgets.Shell(self)
-        self.aui_manager.AddPane(
-            self.shell,
-            aui.AuiPaneInfo()\
-            .Right().Row(0)\
-            .BestSize(400, 600)\
-            .Caption(self.shell.get_uppercase_name())
-            .MaximizeButton(True)\
-            .CloseButton(False)
-        )
-        
-        """
-        self.state_repr_viewer = workspace_widgets.StateReprViewer(self)
-        self.aui_manager.AddPane(
-            self.state_repr_viewer,
-            aui.AuiPaneInfo()\
-            .BestSize(300, 300)\
-            .MaximizeButton(True)\
-            .Center()\
-            .Caption(self.state_repr_viewer.get_uppercase_name())
-            .Floatable(False)\
-            .CloseButton(False)
-        )
-        """
-        settings_wx = self.gui_project.simpack_wx_grokker.settings
-        
-
-        big_widget_class = settings_wx.BIG_WORKSPACE_WIDGETS[0] if \
-                         settings_wx.BIG_WORKSPACE_WIDGETS else \
-                         workspace_widgets.StateReprViewer
-
-        self.big_widget = big_widget_class(self)
-        self.aui_manager.AddPane(
-            self.big_widget,
-            aui.AuiPaneInfo()\
-            .BestSize(300, 300)\
-            .MaximizeButton(True)\
-            .Center()\
-            .Caption(self.big_widget.get_uppercase_name())
-            .Floatable(False)\
-            .CloseButton(False)
-        )
-        
-        if isinstance(self.big_widget, workspace_widgets.StateReprViewer):
-            self.state_repr_viewer = self.big_widget
-        
-        """
-        big_widget_classes = \
-            settings_wx.BIG_WORKSPACE_WIDGETS #+ \
-        #    [workspace_widgets['StateReprViewer']]
-        
-        self.big_widgets = []
-        # todo: not the right way, should be easy listing of all widget
-        
-        
-        for i, BigWidget in enumerate(big_widget_classes):
-            big_widget = BigWidget(self)
+        with wx_tools.WindowFreezer(self):
+            
+            self.tree_browser = workspace_widgets.TreeBrowser(self)
             self.aui_manager.AddPane(
-                big_widget,
+                self.tree_browser,
+                aui.AuiPaneInfo()\
+                .Bottom().Row(0)\
+                .BestSize(1000, 100).MinSize(200, 50).MaxSize(10000, 250)\
+                .Caption(self.tree_browser.get_uppercase_name())
+                .Floatable(False)\
+                .CloseButton(False)
+            )
+            
+            self.playback_controls = workspace_widgets.PlaybackControls(self)
+            self.aui_manager.AddPane(
+                self.playback_controls,
+                aui.AuiPaneInfo()\
+                .Bottom()\
+                .BestSize(184, 128).MinSize(184, 128).MaxSize(184, 128)\
+                .Caption(self.playback_controls.get_uppercase_name())
+                .Resizable(False)\
+                .CloseButton(False)\
+                .Gripper(True)
+            )
+            
+            self.seek_bar = workspace_widgets.SeekBar(self)
+            self.aui_manager.AddPane(
+                self.seek_bar,
+                aui.AuiPaneInfo()\
+                .Bottom().Row(1)\
+                .BestSize(600, 40).MinSize(200, 40).MaxSize(10000, 100)\
+                .Caption(self.seek_bar.get_uppercase_name())
+                .Floatable(False)\
+                .CloseButton(False)
+            )
+            
+            self.shell = workspace_widgets.Shell(self)
+            self.aui_manager.AddPane(
+                self.shell,
+                aui.AuiPaneInfo()\
+                .Right().Row(0)\
+                .BestSize(400, 600)\
+                .Caption(self.shell.get_uppercase_name())
+                .MaximizeButton(True)\
+                .CloseButton(False)\
+                .Gripper(True)
+            )
+            
+            
+            self.crunching_controls = workspace_widgets.CrunchingControls(self)
+            self.aui_manager.AddPane(
+                self.crunching_controls,
+                aui.AuiPaneInfo()\
+                .Left().Row(0)\
+                .BestSize(280, 600)\
+                .Caption(self.crunching_controls.get_uppercase_name())
+                .CloseButton(True)\
+                .Gripper(True)
+            )
+            
+            """
+            self.state_repr_viewer = workspace_widgets.StateReprViewer(self)
+            self.aui_manager.AddPane(
+                self.state_repr_viewer,
                 aui.AuiPaneInfo()\
                 .BestSize(300, 300)\
                 .MaximizeButton(True)\
                 .Center()\
-                .Caption(big_widget.get_uppercase_name())\
+                .Caption(self.state_repr_viewer.get_uppercase_name())
                 .Floatable(False)\
-                .CloseButton(False),
-                target=self.state_repr_viewer.get_aui_pane_info()
+                .CloseButton(False)
             )
-            #.NotebookPage(notebook_id, i)\
-            self.big_widgets.append(big_widget)
-
-        """
-        
-        self.aui_manager.Update()
+            """
+            settings_wx = self.gui_project.simpack_wx_grokker.settings
+            
+    
+            big_widget_class = settings_wx.BIG_WORKSPACE_WIDGETS[0] if \
+                             settings_wx.BIG_WORKSPACE_WIDGETS else \
+                             workspace_widgets.StateReprViewer
+    
+            self.big_widget = big_widget_class(self)
+            self.aui_manager.AddPane(
+                self.big_widget,
+                aui.AuiPaneInfo()\
+                .BestSize(300, 300)\
+                .MaximizeButton(True)\
+                .Center()\
+                .Caption(self.big_widget.get_uppercase_name())
+                .Floatable(False)\
+                .CloseButton(False)
+            )
+            
+            if isinstance(self.big_widget, workspace_widgets.StateReprViewer):
+                self.state_repr_viewer = self.big_widget
+            
+            """
+            big_widget_classes = \
+                settings_wx.BIG_WORKSPACE_WIDGETS #+ \
+            #    [workspace_widgets['StateReprViewer']]
+            
+            self.big_widgets = []
+            # todo: not the right way, should be easy listing of all widget
+            
+            
+            for i, BigWidget in enumerate(big_widget_classes):
+                big_widget = BigWidget(self)
+                self.aui_manager.AddPane(
+                    big_widget,
+                    aui.AuiPaneInfo()\
+                    .BestSize(300, 300)\
+                    .MaximizeButton(True)\
+                    .Center()\
+                    .Caption(big_widget.get_uppercase_name())\
+                    .Floatable(False)\
+                    .CloseButton(False),
+                    target=self.state_repr_viewer.get_aui_pane_info()
+                )
+                #.NotebookPage(notebook_id, i)\
+                self.big_widgets.append(big_widget)
+    
+            """
+            
+            if gui_project._temp_shell_history is not None:
+                assert isinstance(gui_project._temp_shell_history, basestring)
+                self.shell.SetText('')
+                self.shell.write(gui_project._temp_shell_history)
+                self.shell.push('')
+                gui_project._temp_shell_history = None
+                
+            if gui_project._temp_shell_command_history is not None:
+                assert isinstance(gui_project._temp_shell_command_history,
+                                  list)
+                self.shell.history = gui_project._temp_shell_command_history[:]
+                gui_project._temp_shell_command_history = None
+            
+                
+                
+            self.aui_manager.Update()
         
         self.gui_project.emitter_system.top_emitter.emit()
         
@@ -516,15 +586,17 @@ class Frame(wx.Frame):
                 main_script = os.path.abspath(sys.argv[0])
                 program_to_run = [sys.executable, main_script]
                 we_are_main_program = ('run_gui' in main_script) or \
-                                    ('garlicsim_wx' in main_script)
+                                    ('garlicsim_wx' in main_script) or \
+                                    ('GarlicSim' in main_script)
             
             if not we_are_main_program:
                 dialog = \
                     garlicsim_wx.widgets.misc.NotMainProgramWarningDialog(self)
-                if dialog.ShowModal() != wx.ID_YES:
-                    return
-                
-        wildcard = 'GarlicSim Simulation Pickle (*.gssp)|*.gssp|All files (*)|*|'
+                try:
+                    if dialog.ShowModal() != wx.ID_YES:
+                        return
+                finally:
+                    dialog.Destroy()
         
         # Todo: something more sensible here. Ideally should be last place you
         # saved in, but for starters can be desktop.
@@ -534,25 +606,32 @@ class Frame(wx.Frame):
 
         open_dialog = wx.FileDialog(self, message='Choose a file',
                                     defaultDir=folder, defaultFile='',
-                                    wildcard=wildcard, style=wx.OPEN)
-        if open_dialog.ShowModal() == wx.ID_OK:
-            path = open_dialog.GetPath()
-            
-            if self.gui_project is None:
-                self._open_gui_project_from_path(path)
-            else:
-                if hasattr(sys, 'frozen'):
-                    program = [sys.executable]
+                                    wildcard=wildcard_text, style=wx.OPEN)
+        try:
+            if open_dialog.ShowModal() == wx.ID_OK:
+                path = open_dialog.GetPath()
+                
+                if self.gui_project is None:
+                    self._open_gui_project_from_path(path)
                 else:
-                    program = [sys.executable, os.path.abspath(sys.argv[0])]
-                    # Todo: what if some other program is launching my code?
-                    
-                program.append(path)
-             
-                subprocess.Popen(program)
+                    if hasattr(sys, 'frozen'):
+                        program = [sys.executable]
+                    else:
+                        program = \
+                            [sys.executable, os.path.abspath(sys.argv[0])]
                         
-        
-    
+                    program.append(path)
+            
+                    for simpack_place in garlicsim_wx.simpack_places:
+                        program.append('__garlicsim_wx_simpack_place=%s' % \
+                                       ','.join(simpack_place))
+                    
+                    with wx_tools.CursorChanger(self, wx.CURSOR_WAIT):
+                        subprocess.Popen(program)
+        finally:
+            open_dialog.Destroy()
+                        
+            
     def _open_gui_project_from_path(self, path):
         '''
         Open a gui project saved to a file specified by `path`.
@@ -560,38 +639,26 @@ class Frame(wx.Frame):
         Internal use.
         '''
         
-        old_recursion_limit = sys.getrecursionlimit()
-            
-        try:
-            sys.setrecursionlimit(10000)
-            with open(path, 'rb') as my_file:
-                gui_project_vars = pickle_module.load(my_file)
-                
-        except Exception, exception:
-            dialog = wx.MessageDialog(
-                self,
-                'Error opening file:\n' + traceback.format_exc(),
-                style=(wx.OK | wx.ICON_ERROR)
-            )
-            dialog.ShowModal()
-            return
-        
-        finally:
-            sys.setrecursionlimit(old_recursion_limit)
-                
-        if gui_project_vars:
+        with TempRecursionLimitSetter(10000):
             try:
-                gui_project = GuiProject.load_from_vars(self, gui_project_vars)
+                with open(path, 'rb') as my_file:
+                    with wx_tools.CursorChanger(self, wx.CURSOR_WAIT):
+                        unpickler = misc.pickling.Unpickler(my_file)
+                        gui_project = unpickler.load()
+                
             except Exception, exception:
-                error_dialog = wx.MessageDialog(
+                dialog = wx.MessageDialog(
                     self,
                     'Error opening file:\n' + traceback.format_exc(),
                     style=(wx.OK | wx.ICON_ERROR)
                 )
-                error_dialog.ShowModal()
-                error_dialog.Destroy()
-                
-            self.__setup_gui_project(gui_project)
+                try:
+                    dialog.ShowModal()
+                finally:
+                    dialog.Destroy()
+                return
+                        
+        self.__setup_gui_project(gui_project)
 
     
     
@@ -599,38 +666,44 @@ class Frame(wx.Frame):
         '''Raise a dialog for saving a gui project to file.'''
         
         assert self.gui_project is not None
-        wildcard = 'GarlicSim Simulation Pickle (*.gssp)|*.gssp|All files (*)|*|'
+        
         folder = os.getcwd()
         
         save_dialog = wx.FileDialog(self, message='Save file as...',
-                                 defaultDir=folder, defaultFile='',
-                                 wildcard=wildcard,
-                                 style=wx.SAVE | wx.OVERWRITE_PROMPT)
-        if save_dialog.ShowModal() == wx.ID_OK:
-            path = save_dialog.GetPath()
+                                    defaultDir=folder, defaultFile='',
+                                    wildcard=wildcard_text,
+                                    style=wx.SAVE | wx.OVERWRITE_PROMPT)
+        try:
+            if save_dialog.ShowModal() == wx.ID_OK:
+                path = save_dialog.GetPath()
+                
+                # Adding extension if got a plain file because wxPython doesn't
+                # give the checkbox that's supposed to do it on Mac:
+                path = misc_tools.add_extension_if_plain(path, '.gssp')
+                
+                
+                with TempRecursionLimitSetter(10000):
+                    try:
+                        with open(path, 'wb') as my_file:
+                            with wx_tools.CursorChanger(self, wx.CURSOR_WAIT):
+                                with self.gui_project.project.tree.lock.read:
+                                    pickler = misc.pickling.Pickler(
+                                        my_file,
+                                        protocol=2,
+                                    )
+                                    pickler.dump(self.gui_project)
+        
+                    except Exception, exception:
+                        error_dialog = wx.MessageDialog(
+                            self,
+                            'Error saving to file:\n' + traceback.format_exc(),
+                            style=(wx.OK | wx.ICON_ERROR)
+                        )
+                        error_dialog.ShowModal()
+                        error_dialog.Destroy()
             
-            old_recursion_limit = sys.getrecursionlimit()
-            
-            try:
-                sys.setrecursionlimit(10000)
-                with open(path, 'wb') as my_file:
-                    picklable_vars = self.gui_project.__getstate__()
-                    pickle_module.dump(picklable_vars, my_file, protocol=2)
-
-            except Exception, exception:
-                error_dialog = wx.MessageDialog(
-                    self,
-                    'Error saving to file:\n' + traceback.format_exc(),
-                    style=(wx.OK | wx.ICON_ERROR)
-                )
-                error_dialog.ShowModal()
-                error_dialog.Destroy()
-            
-            finally:
-                sys.setrecursionlimit(old_recursion_limit)
-            
-            
-        save_dialog.Destroy()
+        finally:
+            save_dialog.Destroy()
     
     """    
     def delete_gui_project(self,gui_project):
